/*
 * updown.c
 *
 * Created: 2013-04-23 06:23:06
 *  Author: dsosnowski
 */ 


/*****************************
UPDOWN DS
******************************/

#define F_CPU			8000000

//---includes-------------------
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "wdog.h"


#define DISP_PORT	PORTD 
#define DISP_DDR	DDRD
#define DISP_A		6		
#define DISP_B		3
#define DISP_C		1
#define DISP_D		0
#define DISP_E		2
#define DISP_F		5
#define DISP_G		4

#define DP_PORT		PORTB
#define DP_DDR		DDRB
#define DP_DISP		0

#define ANODES_PORT PORTB
#define ANODES_DDR	DDRB
#define ANODE_1 	3
#define ANODE_2 	4
#define ANODE_3 	1
#define ANODE_4 	2

#define BUTTON_PORT	PORTB
#define BUTTON_PIN 	PINB
#define RESET_B		7
#define UP_B		6
#define DOWN_B		5

//---mains---
#define MAIN_DELAY		10
#define DISP_REFRESH_FREQ	62	//~500Hz
#define ACTION_IDLE		0
#define ACTION_RESET	1
#define ACTION_UP		2
#define ACTION_DOWN		3
#define ACTION_SAVE		4

#define NORMAL_DELAY	5
#define LONG_DELAY		250

//---default---
#define MINUS_DIGIT		11
#define NULL			12

#define SET_DP			DP_PORT &= ~(1<<DP_DISP)
#define OFF_DP			DP_PORT |= (1<<DP_DISP)

//0...9, 10, -, 
static char digits[13] = {0x10, 0x75, 0x22, 0x24, 0x45, 0x8C, 0x88, 0x35, 0x00, 0x04, 0x10, 0xEF, 0xFF}; 

//-------------------------
void SwitchScan(void);
void DispalyValue(void);
void RefresfDisplay(void);
void HardInit(void);

//-------------------------
static volatile char display[4];
static volatile char sync;

//char refresh_counter;
static unsigned int main_val;
static char action;
static char switch_delay;
static char reset_delay;


//-------------------------
int main(void)
{
	char temp;

	HardInit();	
	
	_delay_ms(100);
	temp = ee_read(5);
	main_val = temp;
	main_val <<= 8;
	temp = ee_read(6);
	main_val += temp;
	if (main_val > 9999) main_val = 0;
	
	WdtSet(7);

	display[0] = digits[MINUS_DIGIT];
	display[1] = digits[MINUS_DIGIT];
	display[2] = digits[MINUS_DIGIT];
	display[3] = digits[MINUS_DIGIT];
	
	
	//----------------------------------
	while(1)
	{		
		
		SwitchScan();
		OFF_DP;
		if (action == ACTION_UP)
		{
			if (main_val < 9999) 
			{
				main_val++;
				SET_DP;
			}			
							
		}
		if (action == ACTION_DOWN)
		{
			if (main_val > 0)
			{
				main_val--;				
				SET_DP;
			}			
		}
		if (action == ACTION_RESET)
		{
			main_val = 0;
		}
		
		if (action == ACTION_SAVE)
		{
			temp = 	(main_val & 0xFF);
			action = ((main_val>>8) & 0xFF);
			display[0] = digits[NULL];
			display[1] = digits[NULL];
			display[2] = digits[NULL];
			display[3] = digits[NULL];
			DISP_PORT = digits[NULL];
			cli();
			ee_write(5, action);
			ee_write(6, temp);
			sei();		
			_delay_ms(100);
		}
		
		action = ACTION_IDLE;
		
		
		DispalyValue();
					
		//---main delay------------------------------
		WdtReset();
		_delay_ms(MAIN_DELAY);
	}
}

//-------------------------
void SwitchScan(void)
{
	char temp;
	temp = PINB;
	temp >>= 5;
	
	action = ACTION_IDLE;
	
	if (temp < 7)		//any switch
	{
		if (temp == 6) action = ACTION_DOWN;
		if (temp == 5) action = ACTION_UP;
		if (temp == 3) action = ACTION_RESET;
	}
	else
	{
		if ((reset_delay > NORMAL_DELAY) && (reset_delay < (LONG_DELAY/2)))
		{
			action = ACTION_SAVE;
		}		
		reset_delay	= 0;
		switch_delay = 0;		
	}	
	
	if (action)
	{
		if ((action == ACTION_UP) ||(action == ACTION_DOWN))
		{
			if (switch_delay < 0xFF) switch_delay++;
			if (switch_delay != NORMAL_DELAY) action = ACTION_IDLE;			
		}
	
		if (action == ACTION_RESET)
		{
			if (reset_delay < 0xFF) reset_delay++;
			if (reset_delay != LONG_DELAY) action = ACTION_IDLE;									
		}
	}
}



//-------------------------
void DispalyValue(void)
{
	unsigned int temp = 1000;
	char fill_counter = 0;
	char temp_digits[4];
	char digit;
	unsigned int val = main_val;

	//-----------------------------
	while (fill_counter < 4)
	{
		digit = val / temp;
		val = val % temp;
		temp = temp / 10;
		temp_digits[fill_counter] = digit;
		fill_counter++;
	}
	
	if (temp_digits[0] == 0)
	{
		temp_digits[0] = NULL;
		if (temp_digits[1] == 0)
		{
			temp_digits[1] = NULL;
			if (temp_digits[2] == 0)
			{
				temp_digits[2] = NULL;	
			} 
		} 
	} 
		
	digit = temp_digits[0];
	display[3] = digits[digit];		
	digit = temp_digits[1];
	display[2] = digits[digit];		
	digit = temp_digits[2];
	display[1] = digits[digit];		
	digit = temp_digits[3];
	display[0] = digits[digit];		
}

//-------------------------
void RefresfDisplay(void)
{
	static char anode;
		
	if (anode < 5) anode++;
	else anode = 0;
	
	switch (anode)
	{
		case 0x00:
			ANODES_PORT |= (1<<ANODE_1);
			DISP_PORT = display[0];
			ANODES_PORT &= ~(1<<ANODE_4);	
		break;
		case 0x01:
			ANODES_PORT |= (1<<ANODE_4);
			//DP_PORT &= ~(1<<DP_DISP);
			DISP_PORT = display[1];
			ANODES_PORT &= ~(1<<ANODE_3);	
		break;
		case 0x02:
			ANODES_PORT |= (1<<ANODE_3);
			//DP_PORT |= (1<<DP_DISP);
			DISP_PORT = display[2];
			ANODES_PORT &= ~(1<<ANODE_2);	
		break;
		case 0x03:
			ANODES_PORT |= (1<<ANODE_2);
			DISP_PORT = display[3];
			ANODES_PORT &= ~(1<<ANODE_1);	
		break;
		case 0x04:
			ANODES_PORT |= (1<<ANODE_1);
			sync = 1;
		break;					
	}
}

//-------------------------
void HardInit(void)
{
	DISP_PORT = 0xFF;
	DISP_DDR = 0xFF;

	ANODES_PORT = (1<<ANODE_1 | 1<<ANODE_2 | 1<<ANODE_3 | 1<<ANODE_4);
	ANODES_DDR = (1<<ANODE_1 | 1<<ANODE_2 | 1<<ANODE_3 | 1<<ANODE_4 | 1<<DP_DISP);

	BUTTON_PORT |= (1<<UP_B | 1<<DOWN_B | 1<<RESET_B);

	TCCR1A = (1<<WGM10 | 1<<WGM11);
	TCCR1B = (1<<WGM12 | 1<<WGM13 | 1<<CS12);		//div 256
	OCR1A = DISP_REFRESH_FREQ;
	TIMSK = (1<<TOIE1);
	sei();
}

//-----------------------------------
SIGNAL(TIMER1_OVF_vect)
{
	RefresfDisplay();
}


